@@ -71,6 +71,7 @@ struct _EphyShell {
   EphyShellStartupContext *local_startup_context;
   EphyShellStartupContext *remote_startup_context;
   GSList *open_uris_idle_ids;
+  EphyWebApplication *webapp;
 
   gchar *open_notification_id;
   gboolean startup_finished;
@@ -78,6 +79,8 @@ struct _EphyShell {
 #if USE_GRANITE
   GtkStyleProvider *style_provider;
 #endif
+
+  GList *windows; /* unowned, used for assertion only */
 };
 
 static EphyShell *ephy_shell = NULL;
@@ -85,7 +88,7 @@ static EphyShell *ephy_shell = NULL;
 static void ephy_shell_dispose (GObject *object);
 static void ephy_shell_finalize (GObject *object);
 
-G_DEFINE_TYPE (EphyShell, ephy_shell, EPHY_TYPE_EMBED_SHELL)
+G_DEFINE_FINAL_TYPE (EphyShell, ephy_shell, EPHY_TYPE_EMBED_SHELL)
 
 /**
  * ephy_shell_startup_context_new:
@@ -385,6 +388,7 @@ static GActionEntry non_incognito_extra_app_entries[] = {
 };
 
 static GActionEntry app_mode_app_entries[] = {
+  { "new-window", new_window, NULL, NULL, NULL },
   { "history", show_history, NULL, NULL, NULL },
   { "preferences", show_preferences, NULL, NULL, NULL },
   { "about", show_about, NULL, NULL, NULL },
@@ -392,6 +396,7 @@ static GActionEntry app_mode_app_entries[] = {
   { "run-in-background", NULL, NULL, "false", NULL},
 };
 
+#if 0
 static void
 register_synchronizable_managers (EphyShell       *shell,
                                   EphySyncService *service)
@@ -406,10 +411,12 @@ register_synchronizable_managers (EphyShell       *shell,
     ephy_sync_service_register_manager (service, manager);
   }
 
+/* https://gitlab.gnome.org/GNOME/epiphany/-/issues/1118#note_1731178
   if (ephy_sync_utils_bookmarks_sync_is_enabled ()) {
     manager = EPHY_SYNCHRONIZABLE_MANAGER (ephy_shell_get_bookmarks_manager (shell));
     ephy_sync_service_register_manager (service, manager);
   }
+*/
 
   if (ephy_sync_utils_passwords_sync_is_enabled ()) {
     manager = EPHY_SYNCHRONIZABLE_MANAGER (ephy_embed_shell_get_password_manager (EPHY_EMBED_SHELL (shell)));
@@ -457,6 +464,7 @@ sync_secrets_load_finished_cb (EphySyncService *service,
   register_synchronizable_managers (shell, service);
   ephy_sync_service_start_sync (service);
 }
+#endif
 
 static void
 set_accel_for_action (EphyShell   *shell,
@@ -553,7 +561,6 @@ ephy_shell_startup (GApplication *application)
     }
 
     /* Actions that are disabled in app mode */
-    set_accel_for_action (shell, "app.new-window", "<Primary>n");
     set_accel_for_action (shell, "app.new-incognito", "<Primary><Shift>n");
     set_accel_for_action (shell, "app.reopen-closed-tab", "<Primary><Shift>t");
     set_accel_for_action (shell, "app.import-bookmarks", "<Primary><Shift>m");
@@ -561,6 +568,8 @@ ephy_shell_startup (GApplication *application)
     set_accel_for_action (shell, "app.shortcuts", "<Primary>question");
     set_accel_for_action (shell, "app.help", "F1");
   } else {
+    shell->webapp = ephy_web_application_for_profile_directory (ephy_profile_dir (), EPHY_WEB_APP_NO_TMP_ICON);
+
     g_action_map_add_action_entries (G_ACTION_MAP (application),
                                      app_mode_app_entries, G_N_ELEMENTS (app_mode_app_entries),
                                      application);
@@ -578,6 +587,7 @@ ephy_shell_startup (GApplication *application)
   }
 
   /* Actions that are available in both app mode and browser mode */
+  set_accel_for_action (shell, "app.new-window", "<Primary>n");
   set_accel_for_action (shell, "app.history", "<Primary>h");
   set_accel_for_action (shell, "app.preferences", "<Primary>comma");
   set_accel_for_action (shell, "app.quit", "<Primary>q");
@@ -884,6 +894,20 @@ remove_open_uris_idle_cb (gpointer data)
   g_source_remove (GPOINTER_TO_UINT (data));
 }
 
+void
+ephy_shell_register_window (EphyShell  *shell,
+                            EphyWindow *window)
+{
+  shell->windows = g_list_prepend (shell->windows, window);
+}
+
+void
+ephy_shell_unregister_window (EphyShell  *shell,
+                              EphyWindow *window)
+{
+  shell->windows = g_list_remove (shell->windows, window);
+}
+
 static void
 ephy_shell_dispose (GObject *object)
 {
@@ -905,6 +929,7 @@ ephy_shell_dispose (GObject *object)
   g_clear_object (&shell->history_manager);
   g_clear_object (&shell->open_tabs_manager);
   g_clear_object (&shell->web_extension_manager);
+  g_clear_pointer (&shell->webapp, ephy_web_application_free);
 
   if (shell->open_notification_id) {
     g_application_withdraw_notification (G_APPLICATION (shell), shell->open_notification_id);
@@ -929,6 +954,9 @@ ephy_shell_finalize (GObject *object)
   g_clear_pointer (&shell->local_startup_context, ephy_shell_startup_context_free);
   g_clear_pointer (&shell->remote_startup_context, ephy_shell_startup_context_free);
 
+  /* Ensure all windows have been destroyed. */
+  g_assert (!shell->windows);
+
   G_OBJECT_CLASS (ephy_shell_parent_class)->finalize (object);
 
   LOG ("Ephy shell finalised");
@@ -1133,6 +1161,9 @@ ephy_shell_get_sync_service (EphyShell *shell)
 {
   g_assert (EPHY_IS_SHELL (shell));
 
+#if 0
+  /* Firefox Sync is disabled due to: https://gitlab.gnome.org/GNOME/epiphany/-/issues/2337 */
+
   if (shell->sync_service == NULL) {
     shell->sync_service = ephy_sync_service_new (TRUE);
 
@@ -1145,6 +1176,7 @@ ephy_shell_get_sync_service (EphyShell *shell)
                              G_CALLBACK (sync_secrets_load_finished_cb),
                              shell, 0);
   }
+#endif
 
   return shell->sync_service;
 }
@@ -1598,3 +1630,37 @@ ephy_shell_get_active_web_view (EphyShell *shell)
 
   return ephy_embed_get_web_view (EPHY_EMBED (page));
 }
+
+EphyWebApplication *
+ephy_shell_get_webapp (EphyShell *shell)
+{
+  return shell->webapp;
+}
+
+void
+ephy_shell_resync_title_boxes (EphyShell  *shell,
+                               const char *title,
+                               const char *address)
+{
+  GList *windows;
+  EphyEmbedShellMode mode;
+
+  mode = ephy_embed_shell_get_mode (EPHY_EMBED_SHELL (shell));
+  g_assert (mode == EPHY_EMBED_SHELL_MODE_APPLICATION);
+
+  windows = gtk_application_get_windows (GTK_APPLICATION (shell));
+  while (windows) {
+    EphyWindow *window = EPHY_WINDOW (windows->data);
+    EphyHeaderBar *header_bar;
+    EphyTitleBox *title_box;
+
+    header_bar = EPHY_HEADER_BAR (ephy_window_get_header_bar (window));
+    title_box = EPHY_TITLE_BOX (ephy_header_bar_get_title_widget (header_bar));
+    g_assert (EPHY_IS_TITLE_BOX (title_box));
+
+    ephy_title_box_reset (title_box, title, address);
+    gtk_window_set_title (GTK_WINDOW (window), title);
+
+    windows = windows->next;
+  }
+}
